#ifndef PCD8544_H_
#define PCD8544_H_

#include <stdint.h>

#define PCD8544_ADDRESSING_VERTICAL	0x02
#define PCD8544_ADDRESSING_HORIZONTAL	0x00

#define PCD8544_VIDEO_MODE_BLANK	0x00
#define PCD8544_VIDEO_MODE_NORMAL	0x04
#define PCD8544_VIDEO_MODE_ALL_ON	0x01
#define PCD8544_VIDEO_MODE_INVERSE	0x05

#define PCD8544_PIN_BANK		DDRB
#define PCD8544_PIN_PORT		PORTB
#define PCD8544_VCC_BANK		DDRD
#define PCD8544_VCC_PORT		PORTD

#define PCD8544_RST			PB0
#define PCD8544_SCE			PB1
#define PCD8544_DC			PB2
#define PCD8544_SDIN			PB5
#define PCD8544_SCLK			PB7
#define PCD8544_VCC			PD0

#define PCD8544_MAX_X			83
#define PCD8544_MAX_Y			5

#define PCD8544_DEFAULT_VOP		0x45

/*
 * Initializes the display.
 */
void PCD8544_init();

/*
 * Clears the display.
 */
void PCD8544_clear();

/*
 * Displays data.
 */
void PCD8544_data(const uint8_t data);

/*
 * Displays a stream of pixels.
 */
void PCD8544_stream(const uint8_t data[], const uint8_t len);

/*
 * Transmits a raw command to the display.
 *
 * This allows for low level access to all the display options.
 */
void PCD8544_cmd(const uint8_t cmd);

/*
 * Moves the display cursor to the specified position.
 */
void PCD8544_goto(const uint8_t x, const uint8_t y);

/*
 * Returns the current value for the operational display voltage.
 */
uint8_t PCD8544_get_vop();

/*
 * Sets a new value for the operational display voltage.
 */
void PCD8544_set_vop(const uint8_t vop);

/*
 * Changes the addressing mode of the display.
 *
 * Valid choices are PCD8544_ADDRESSING_VERTICAL for vertical addressing and
 * PCD8544_ADDRESSING_HORIZONTAL for horizontal addressing.
 *
 * The behaviour is undefined for other values.
 */
void PCD8544_addr_mode(const uint8_t addr_mode);

/*
 * Sets the video mode of the display.
 *
 * Valid choices are PCD8544_VIDEO_MODE_BLANK for a blank display,
 * PCD8544_VIDEO_MODE_NORMAL for normal video mode, PCD8544_VIDEO_MODE_ALL_ON
 * for having all the pixels on and PCD8544_VIDEO_MODE_INVERSE for having the
 * pixels you write turned off and the rest of the display turned on.
 *
 * The behaviour is undefined for other values.
 */
void PCD8544_video_mode(const uint8_t video_mode);

#endif
