#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define TEMPO		28
#define PAUSE_LEN	20
#define PAUSE		0

#define NOTE_B0  31
#define NOTE_C1  33
#define NOTE_CS1 35
#define NOTE_D1  37
#define NOTE_DS1 39
#define NOTE_E1  41
#define NOTE_F1  44
#define NOTE_FS1 46
#define NOTE_G1  49
#define NOTE_GS1 52
#define NOTE_A1  55
#define NOTE_AS1 58
#define NOTE_B1  62
#define NOTE_C2  65
#define NOTE_CS2 69
#define NOTE_D2  73
#define NOTE_DS2 78
#define NOTE_E2  82
#define NOTE_F2  87
#define NOTE_FS2 93
#define NOTE_G2  98
#define NOTE_GS2 104
#define NOTE_A2  110
#define NOTE_AS2 117
#define NOTE_B2  123
#define NOTE_C3  131
#define NOTE_CS3 139
#define NOTE_D3  147
#define NOTE_DS3 156
#define NOTE_E3  165
#define NOTE_F3  175
#define NOTE_FS3 185
#define NOTE_G3  196
#define NOTE_GS3 208
#define NOTE_A3  220
#define NOTE_AS3 233
#define NOTE_B3  247
#define NOTE_C4  262
#define NOTE_CS4 277
#define NOTE_D4  294
#define NOTE_DS4 311
#define NOTE_E4  330
#define NOTE_F4  349
#define NOTE_FS4 370
#define NOTE_G4  392
#define NOTE_GS4 415
#define NOTE_A4  440
#define NOTE_AS4 466
#define NOTE_B4  494
#define NOTE_C5  523
#define NOTE_CS5 554
#define NOTE_D5  587
#define NOTE_DS5 622
#define NOTE_E5  659
#define NOTE_F5  698
#define NOTE_FS5 740
#define NOTE_G5  784
#define NOTE_GS5 831
#define NOTE_A5  880
#define NOTE_AS5 932
#define NOTE_B5  988
#define NOTE_C6  1047
#define NOTE_CS6 1109
#define NOTE_D6  1175
#define NOTE_DS6 1245
#define NOTE_E6  1319
#define NOTE_F6  1397
#define NOTE_FS6 1480
#define NOTE_G6  1568
#define NOTE_GS6 1661
#define NOTE_A6  1760
#define NOTE_AS6 1865
#define NOTE_B6  1976
#define NOTE_C7  2093
#define NOTE_CS7 2217
#define NOTE_D7  2349
#define NOTE_DS7 2489
#define NOTE_E7  2637
#define NOTE_F7  2794
#define NOTE_FS7 2960
#define NOTE_G7  3136
#define NOTE_GS7 3322
#define NOTE_A7  3520
#define NOTE_AS7 3729
#define NOTE_B7  3951
#define NOTE_C8  4186
#define NOTE_CS8 4435
#define NOTE_D8  4699
#define NOTE_DS8 4978

/*
 * Stops note playing by disabling the PWM note timer and the output pin.
 */
#define STOP_NOTE() 					\
	do {						\
		TCCR0B &= ~(7 << CS00);			\
		TCCR0A &= ~(1 << COM0B1);		\
	} while (0)

#define START_NOTE()					\
	do {						\
		TCCR0A |= (1 << COM0B1);		\
		TCCR0B |= (1 << CS02);			\
	} while (0)

#define STOP_DURATION()		(TCCR1B &= ~(7 << CS10))

#define START_DURATION() 	(TCCR1B |= (1 << CS12) | (1 << CS10))

/*
 * Sets the duration of a note/pause by modifying the TIMER1 top.
 */
#define SET_DURATION(d)	(OCR1A = F_CPU / beat_prescaler / 1000 * (d))

uint16_t notes[] = {
  NOTE_E7, NOTE_E7, PAUSE, NOTE_E7,
  PAUSE, NOTE_C7, NOTE_E7, PAUSE,
  NOTE_G7, PAUSE, PAUSE,  PAUSE,
  NOTE_G6, PAUSE, PAUSE, PAUSE,

  NOTE_C7, PAUSE, PAUSE, NOTE_G6,
  PAUSE, PAUSE, NOTE_E6, PAUSE,
  PAUSE, NOTE_A6, PAUSE, NOTE_B6,
  PAUSE, NOTE_AS6, NOTE_A6, PAUSE,

  NOTE_G6, NOTE_E7, NOTE_G7,
  NOTE_A7, PAUSE, NOTE_F7, NOTE_G7,
  PAUSE, NOTE_E7, PAUSE, NOTE_C7,
  NOTE_D7, NOTE_B6, PAUSE, PAUSE,

  NOTE_C7, PAUSE, PAUSE, NOTE_G6,
  PAUSE, PAUSE, NOTE_E6, PAUSE,
  PAUSE, NOTE_A6, PAUSE, NOTE_B6,
  PAUSE, NOTE_AS6, NOTE_A6, PAUSE,

  NOTE_G6, NOTE_E7, NOTE_G7,
  NOTE_A7, PAUSE, NOTE_F7, NOTE_G7,
  PAUSE, NOTE_E7, PAUSE, NOTE_C7,
  NOTE_D7, NOTE_B6, PAUSE, PAUSE
};

uint8_t beats[] = {
  4, 4, 4, 4,
  4, 4, 4, 4,
  4, 4, 4, 4,
  4, 4, 4, 4,

  4, 4, 4, 4,
  4, 4, 4, 4,
  4, 4, 4, 4,
  4, 4, 4, 4,

  3, 3, 3,
  4, 4, 4, 4,
  4, 4, 4, 4,
  4, 4, 4, 4,

  4, 4, 4, 4,
  4, 4, 4, 4,
  4, 4, 4, 4,
  4, 4, 4, 4,

  3, 3, 3,
  4, 4, 4, 4,
  4, 4, 4, 4,
  4, 4, 4, 4,
};

static const int SONG_LEN = sizeof(notes) / (sizeof *notes);

static int note_prescaler;
static int beat_prescaler;

/* Current note. */
static volatile uint16_t note;

/* Have I made a pause between notes? */
static volatile bool paused;

static void play_note(int frequency);

/*
 * Initializes TIMER0 which will be used to generate the PWM pulse to play a
 * note.
 */
static void init_note_pwm_timer()
{
	/* PWM output pin. */
	DDRB |= (1 << PB4);

	/* Fast PWM, top at OCR0A. */
	TCCR0A |= (1 << WGM01) | (1 << WGM00);
	TCCR0B |= (1 << WGM02);

	/* Set OC0B at bottom, clear on compare match. */
	TCCR0A |= (1 << COM0B1);

	/* clkI/O / 256 = 257 to 32768 Hz frequency range. */
	note_prescaler = 256;
}

static void init_duration_timer()
{
	/* CTC, top at OCR1A. */
	TCCR1B |= (1 << WGM12);

	/* Interrupt on compare with OCR1A. */
	TIMSK1 |= (1 << OCIE1A);

	/* Prescaler at 1024 - duration between 0.06ms and 4s. */
	beat_prescaler = 1024;
	SET_DURATION(beats[note] * TEMPO);

	TCCR1B |= (1 << CS12) | (1 << CS10);
}

/*
 * Modifies the PWM pulse to play a note specified by its frequency.
 */
static inline void play_note(int frequency)
{
	OCR0A = F_CPU / note_prescaler / frequency * 1.5;
	OCR0B = OCR0A / 2;

	START_NOTE();
}

/*
 * Starts playing the song.
 */
void start_music()
{
	init_note_pwm_timer();
	init_duration_timer();

	note = 0;
	paused = false;
	play_note(notes[note]);
}

void pause_music()
{
	/* Stop note playing. */
	STOP_NOTE();

	/* Stop song advancing. */
	STOP_DURATION();
}

void resume_music()
{
	play_note(notes[note]);

	/* Start note advancing. */
	START_DURATION();
}

/*
 * Note played, do a pause then proceed to the next note.
 */
ISR(TIMER1_COMPA_vect)
{
	if (!paused) {
		paused = true;
		STOP_NOTE();
		SET_DURATION(PAUSE_LEN);
	} else {
		paused = false;
		note++;
		if (note == SONG_LEN)
			/* Replay the song. */
			note = 0;

		if (notes[note] == PAUSE)
			STOP_NOTE();
		else
			play_note(notes[note]);

		SET_DURATION(beats[note] * TEMPO);
	}
}

#ifdef DEBUG
int main(void)
{
	sei();
	start_music();

	while (1)
		;

	return EXIT_SUCCESS;
}
#endif
