#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>

#define TXD0			PD1
#define RXD0			PD0
#define TXD1			PD3
#define RXD1			PD2

#define BAUD_2400_NORMAL_UBRR	416
#define BAUD_4800_NORMAL_UBRR	207
#define BAUD_9600_NORMAL_UBRR	103
#define BAUD_14400_NORMAL_UBRR	68
#define BAUD_19200_NORMAL_UBRR	51
#define BAUD_28800_NORMAL_UBRR	34
#define BAUD_38400_NORMAL_UBRR	25
#define BAUD_57600_NORMAL_UBRR	16
#define BAUD_76800_NORMAL_UBRR	12
#define BAUD_115200_NORMAL_UBRR	8

void USART0_8N1_init(int32_t baud)
{
	/* TXD0 is output, make it one because a frame starts with zero. */
	DDRD |= (1 << TXD0);
	PORTD |= (1 << TXD0);

	/* RXD0 is input, activate pull-up resistor. */
	DDRD &= ~(1 << RXD0);
	PORTD |= (1 << RXD0);

	/* 8 bit character size, no parity. */
	UCSR0C |= (1 << UCSZ00) | (1 << UCSZ01);

	/* One stop bit. */
	UCSR0C &= ~(1 << USBS0);

	if (baud == 2400)
		UBRR0 = BAUD_2400_NORMAL_UBRR;
	else if (baud == 4800)
		UBRR0 = BAUD_4800_NORMAL_UBRR;
	else if (baud == 9600)
		UBRR0 = BAUD_9600_NORMAL_UBRR;
	else if (baud == 14400)
		UBRR0 = BAUD_14400_NORMAL_UBRR;
	else if (baud == 19200)
		UBRR0 = BAUD_19200_NORMAL_UBRR;
	else if (baud == 28800)
		UBRR0 = BAUD_28800_NORMAL_UBRR;
	else if (baud == 38400)
		UBRR0 = BAUD_38400_NORMAL_UBRR;
	else if (baud == 57600)
		UBRR0 = BAUD_57600_NORMAL_UBRR;
	else if (baud == 76800)
		UBRR0 = BAUD_76800_NORMAL_UBRR;
	else if (baud == 115200)
		UBRR0 = BAUD_115200_NORMAL_UBRR;
	else
		/* Make 115.2k the default. */
		UBRR0 = BAUD_115200_NORMAL_UBRR;

	/* Enable RX and TX. */
	UCSR0B |= (1 << RXEN0) | (1 << TXEN0);
}

void USART0_transmit(char data)
{
	while ((UCSR0A & (1 << UDRE0)) == 0)
		;
	UDR0 = data;
}

void USART0_puts(const char data[])
{
	uint8_t i;

	for (i = 0; data[i] != '\0'; i++) {
		while ((UCSR0A & (1 << UDRE0)) == 0)
			;
		UDR0 = data[i];
	}
}

char USART0_receive()
{
	while ((UCSR0A & (1 << RXC0)) == 0)
		;
	return UDR0;
}

void USART1_8N1_init(int32_t baud)
{
	/* TXD1 is output, make it one because a frame starts with zero. */
	DDRD |= (1 << TXD1);
	PORTD |= (1 << TXD1);

	/* RXD1 is input, activate pull-up resistor. */
	DDRD &= ~(1 << RXD1);
	PORTD |= (1 << RXD1);

	/* 8 bit character size. */
	UCSR1C |= (1 << UCSZ10) | (1 << UCSZ11);

	/* One stop bit, no parity. */
	UCSR1C &= ~(1 << USBS1);

	if (baud == 2400)
		UBRR1 = BAUD_2400_NORMAL_UBRR;
	else if (baud == 4800)
		UBRR1 = BAUD_4800_NORMAL_UBRR;
	else if (baud == 9600)
		UBRR1 = BAUD_9600_NORMAL_UBRR;
	else if (baud == 14400)
		UBRR1 = BAUD_14400_NORMAL_UBRR;
	else if (baud == 19200)
		UBRR1 = BAUD_19200_NORMAL_UBRR;
	else if (baud == 28800)
		UBRR1 = BAUD_28800_NORMAL_UBRR;
	else if (baud == 38400)
		UBRR1 = BAUD_38400_NORMAL_UBRR;
	else if (baud == 57600)
		UBRR1 = BAUD_57600_NORMAL_UBRR;
	else if (baud == 76800)
		UBRR1 = BAUD_76800_NORMAL_UBRR;
	else if (baud == 115200)
		UBRR1 = BAUD_115200_NORMAL_UBRR;
	else
		/* Make 115.2k the default. */
		UBRR1 = BAUD_115200_NORMAL_UBRR;

	/* Enable RX and TX. */
	UCSR1B |= (1 << RXEN1) | (1 << TXEN1);
}

void USART1_transmit(char data)
{
	while ((UCSR1A & (1 << UDRE1)) == 0)
		;
	UDR1 = data;
}

void USART1_puts(const char data[])
{
	uint8_t i;

	for (i = 0; data[i] != '\0'; i++) {
		while ((UCSR1A & (1 << UDRE1)) == 0)
			;
		UDR1 = data[i];
	}
}

char USART1_receive()
{
	while ((UCSR1A & (1 << RXC1)) == 0)
		;
	return UDR1;
}

/*
int main(void)
{
	char data;

	USART1_8N1_init(115200);

	USART1_transmit('h');
	USART1_transmit('e');
	USART1_transmit('l');
	USART1_transmit('l');
	USART1_transmit('o');

	while (1) {
		data = USART1_receive();

		if (data == 'h') {
			USART1_transmit('\n');
			USART1_transmit('h');
			USART1_transmit('e');
			USART1_transmit('l');
			USART1_transmit('l');
			USART1_transmit('o');
			USART1_transmit('\n');
		} else {
			USART1_transmit('\n');
			USART1_transmit('n');
			USART1_transmit('a');
			USART1_transmit('y');
			USART1_transmit('\n');
		}

		USART1_transmit(data);
	}

	return EXIT_SUCCESS;
}
*/
